@@ -1,5 +1,5 @@
 /* ATK - Accessibility Toolkit
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -85,6 +85,18 @@ atk_gobject_accessible_for_object (GObject *obj)
                                           G_OBJECT_TYPE (obj));
       accessible = atk_object_factory_create_accessible (factory,
                                                          obj);
+      if (!ATK_IS_GOBJECT_ACCESSIBLE (accessible))
+        {
+          /*
+           * The AtkObject which was created was not a AtkGObjectAccessible
+           */
+          quark_accessible_object = g_quark_from_static_string ("accessible-object");
+          g_object_weak_ref (obj,
+                             (GWeakNotify) g_object_unref,
+                             accessible); 
+          if (!quark_accessible_object)
+            quark_accessible_object = g_quark_from_static_string ("accessible-object");
+        }
       g_object_set_qdata (obj, quark_accessible_object, accessible);
     }
   return accessible;
@@ -127,7 +139,7 @@ atk_gobject_accessible_dispose (gpointer  data)
 {
   g_return_if_fail (ATK_IS_GOBJECT_ACCESSIBLE (data));
 
-  g_object_set_qdata (G_OBJECT (data), quark_accessible_object, NULL);
+  g_object_set_qdata (G_OBJECT (data), quark_object, NULL);
   atk_object_notify_state_change (ATK_OBJECT (data), ATK_STATE_DEFUNCT,
                                   TRUE); 
   g_object_unref (data);
@@ -144,6 +156,7 @@ atk_gobject_accessible_class_init (AtkGObjectAccessibleClass *klass)
 
   class->initialize = atk_real_gobject_accessible_initialize;
 
-  quark_accessible_object = g_quark_from_static_string ("accessible-object");
+  if (!quark_accessible_object)
+    quark_accessible_object = g_quark_from_static_string ("accessible-object");
   quark_object = g_quark_from_static_string ("object-for-accessible");
 }
